@@ -84,11 +84,13 @@ extern bool flag_files;
 extern bool flag_files_with_matches;
 extern bool flag_files_without_match;
 extern bool flag_fixed_strings;
+extern bool flag_hex;
 extern bool flag_hex_star;
 extern bool flag_hex_cbr;
 extern bool flag_hex_chr;
 extern bool flag_hex_hbr;
 extern bool flag_hidden;
+extern bool flag_hyperlink_line;
 extern bool flag_invert_match;
 extern bool flag_json;
 extern bool flag_line_buffered;
@@ -109,7 +111,6 @@ extern bool flag_stdin;
 extern bool flag_usage_warnings;
 extern bool flag_word_regexp;
 extern bool flag_xml;
-extern bool flag_hex;
 extern bool flag_with_hex;
 extern bool flag_no_filename;
 extern bool flag_with_filename;
@@ -170,16 +171,22 @@ extern const char *flag_format_end;
 extern const char *flag_format_open;
 extern const char *flag_group_separator;
 extern const char *flag_hexdump;
+extern const char *flag_hyperlink;
 extern const char *flag_label;
 extern const char *flag_pager;
 extern const char *flag_replace;
 extern const char *flag_save_config;
 extern const char *flag_separator;
+extern const char *flag_separator_dash;
+extern const char *flag_separator_plus;
+extern const char *flag_separator_bar;
 extern const char *flag_sort;
 extern const char *flag_stats;
 extern const char *flag_tag;
 extern const char *flag_view;
 extern std::string flag_config_file;
+extern std::string flag_hyperlink_prefix;
+extern std::string flag_hyperlink_path;
 extern std::set<std::string> flag_config_options;
 extern std::vector<std::string> flag_regexp;
 extern std::vector<std::string> flag_file;
