@@ -30,7 +30,7 @@
 @file      output.cpp
 @brief     Output management
 @author    Robert van Engelen - engelen@genivia.com
-@copyright (c) 2019-2022, Robert van Engelen, Genivia Inc. All rights reserved.
+@copyright (c) 2019-2023, Robert van Engelen, Genivia Inc. All rights reserved.
 @copyright (c) BSD-3 License - see LICENSE.txt
 */
 
@@ -212,35 +212,56 @@ void Output::Dump::line()
 }
 
 // output the header part of the match, preceding the matched line
-void Output::header(const char *& pathname, const std::string& partname, size_t lineno, reflex::AbstractMatcher *matcher, size_t byte_offset, const char *separator, bool newline)
+void Output::header(const char *pathname, const std::string& partname, bool& heading, size_t lineno, reflex::AbstractMatcher *matcher, size_t byte_offset, const char *separator, bool newline)
 {
   // if hex dump line is incomplete and a header is output, then complete the hex dump first
   if (dump.incomplete() &&
-      ((flag_with_filename && pathname != NULL) ||
+      (heading ||
        (!flag_no_filename && !partname.empty()) ||
        flag_line_number ||
        flag_column_number ||
        flag_byte_offset))
     dump.done();
 
+  // get column number when we need it
+  size_t columno = flag_column_number && matcher != NULL ? matcher->columno() + 1 : 1;
+
+  bool hyp = pathname != LABEL_STANDARD_INPUT && color_hl != NULL; // include hyperlinks
   bool sep = false; // when a separator is needed
-  bool nul = false; // -Q: mark pathname with three NUL bytes unless -a
+  bool nul = false; // -Q: mark pathname with three \0 markers unless -a
 
-  if (flag_with_filename && pathname != NULL)
+  if (heading && flag_query > 0 && !flag_text)
   {
-    nul = flag_query > 0 && !flag_text;
+    nul = true;
+    chr('\0');
+  }
 
-    if (nul)
-      chr('\0');
-
-    str(color_fn);
-    if (color_hl != NULL)
+  // --hyperlink: open link
+  if (hyp)
+  {
+    str(color_hl);
+    str(flag_hyperlink_prefix);
+    str("://");
+    uri(flag_hyperlink_path);
+    chr('/');
+    uri(pathname);
+    if (!(heading && flag_heading) && flag_hyperlink_line)
     {
-      str(color_hl);
-      uri(color_wd);
-      uri(pathname);
-      str(color_st);
+      chr(':');
+      num(lineno);
+      if (flag_column_number)
+      {
+        chr(':');
+        num(columno);
+      }
     }
+    str(color_st);
+  }
+
+  // header should include the pathname
+  if (heading)
+  {
+    str(color_fn);
 
     if (nul)
       chr('\0');
@@ -250,11 +271,6 @@ void Output::header(const char *& pathname, const std::string& partname, size_t
     if (nul)
       chr('\0');
 
-    if (color_hl != NULL)
-    {
-      str(color_hl);
-      str(color_st);
-    }
     str(color_off);
 
     if (flag_null)
@@ -262,11 +278,42 @@ void Output::header(const char *& pathname, const std::string& partname, size_t
 
     if (flag_heading)
     {
+      // --hyperlink: close link
+      if (hyp)
+      {
+        str(color_hl);
+        str(color_st);
+      }
+
       str(color_fn);
       str(color_del);
       str(color_off);
       nl();
-      pathname = NULL;
+
+      // --hyperlink: open link
+      if (hyp)
+      {
+        str(color_hl);
+        str(flag_hyperlink_prefix);
+        str("://");
+        uri(flag_hyperlink_path);
+        chr('/');
+        uri(pathname);
+        if (flag_hyperlink_line)
+        {
+          chr(':');
+          num(lineno);
+          if (flag_column_number)
+          {
+            chr(':');
+            num(columno);
+          }
+        }
+        str(color_st);
+      }
+
+      // the next headers should not include this pathname
+      heading = false;
     }
     else
     {
@@ -324,12 +371,19 @@ void Output::header(const char *& pathname, const std::string& partname, size_t
     }
 
     str(color_cn);
-    num((matcher != NULL ? matcher->columno() + 1 : 1), (flag_initial_tab ? 3 : 1));
+    num(columno, (flag_initial_tab ? 3 : 1));
     str(color_off);
 
     sep = true;
   }
 
+  // --hyperlink: close link
+  if (hyp)
+  {
+    str(color_hl);
+    str(color_st);
+  }
+
   if (flag_byte_offset)
   {
     if (sep)
@@ -360,10 +414,11 @@ void Output::header(const char *& pathname, const std::string& partname, size_t
   }
 }
 
-// output the pathname header for --files_with_matches and --count
+// output the short pathname header for --files_with_matches and --count
 void Output::header(const char *pathname, const std::string& partname)
 {
-  bool nul = flag_query > 0; // -Q: mark pathname with three NUL bytes
+  bool hyp = pathname != LABEL_STANDARD_INPUT && color_hl != NULL; // include hyperlinks
+  bool nul = flag_query > 0; // -Q: mark pathname with three \0 markers for quick navigation
 
   if (flag_tree)
   {
@@ -377,6 +432,7 @@ void Output::header(const char *pathname, const std::string& partname)
       Tree::path.pop_back();
 
       size_t len = Tree::path.rfind(PATHSEPCHR);
+
       if (len == std::string::npos)
         Tree::path.clear();
       else
@@ -390,25 +446,39 @@ void Output::header(const char *pathname, const std::string& partname)
     {
       for (int i = 0; i < Tree::depth; ++i)
         str(Tree::bar);
-      while (--up > 0)
+      for (int i = 1; i < up; ++i)
         str(Tree::end);
       nl();
+
+      // make sure to add a break between trees with terminated leafs
+      if (up > 1 && *Tree::end != '\0' && Tree::depth == 0)
+        nl();
+    }
+    else if (Tree::path.empty() && strchr(pathname, PATHSEPCHR) != NULL)
+    {
+      // add a break between the list of filenames without path and filenames with paths
+      nl();
     }
 
     const char *sep;
 
     while ((sep = strchr(pathname + Tree::path.size(), PATHSEPCHR)) != NULL)
     {
+      if (nul)
+        chr('\0');
+
       for (int i = 1; i < Tree::depth; ++i)
         str(Tree::bar);
+
       if (Tree::depth > 0)
         str(Tree::ptr);
-      else if (nul)
-        str("\0\0", 2);
+
+      if (nul)
+        chr('\0');
 
       str(pathname + Tree::path.size(), sep - (pathname + Tree::path.size()) + 1);
 
-      if (nul && Tree::depth == 0)
+      if (nul)
         chr('\0');
       nl();
 
@@ -416,29 +486,34 @@ void Output::header(const char *pathname, const std::string& partname)
       ++Tree::depth;
     }
 
+    if (nul)
+      chr('\0');
+
     for (int i = 1; i < Tree::depth; ++i)
       str(Tree::bar);
+
     if (Tree::depth > 0)
       str(Tree::ptr);
-    else if (nul)
-      chr('\0');
 
     str(color_fn);
 
-    if (color_hl != NULL)
+    if (hyp)
     {
       str(color_hl);
-      uri(color_wd);
+      str(flag_hyperlink_prefix);
+      str("://");
+      uri(flag_hyperlink_path);
+      chr('/');
       uri(pathname);
       str(color_st);
     }
 
-    if (nul && Tree::depth == 0)
+    if (nul)
       chr('\0');
 
     str(pathname + Tree::path.size());
 
-    if (nul && Tree::depth == 0)
+    if (nul)
       chr('\0');
 
   }
@@ -449,10 +524,13 @@ void Output::header(const char *pathname, const std::string& partname)
 
     str(color_fn);
 
-    if (color_hl != NULL)
+    if (hyp)
     {
       str(color_hl);
-      uri(color_wd);
+      str(flag_hyperlink_prefix);
+      str("://");
+      uri(flag_hyperlink_path);
+      chr('/');
       uri(pathname);
       str(color_st);
     }
@@ -465,12 +543,11 @@ void Output::header(const char *pathname, const std::string& partname)
     if (nul)
       chr('\0');
 
-    if (color_hl != NULL)
+    if (hyp)
     {
       str(color_hl);
       str(color_st);
     }
-
   }
 
   if (!partname.empty())
@@ -488,37 +565,42 @@ void Output::binary_file_matches(const char *pathname, const std::string& partna
 {
   if ((mode_ & BINARY) != 0)
     return;
+
   str(color_off);
-  str("Binary file");
+  str("Binary file ");
   str(color_fn);
-  if (pathname != NULL)
+
+  if (pathname != LABEL_STANDARD_INPUT && color_hl != NULL)
   {
-    chr(' ');
-    if (color_hl != NULL)
-    {
-      str(color_hl);
-      uri(color_wd);
-      uri(pathname);
-      str(color_st);
-    }
-    str(pathname);
-    if (color_hl != NULL)
-    {
-      str(color_hl);
-      str(color_st);
-    }
+    str(color_hl);
+    str(flag_hyperlink_prefix);
+    str("://");
+    uri(flag_hyperlink_path);
+    chr('/');
+    uri(pathname);
+    str(color_st);
   }
+
+  str(pathname);
+
+  if (pathname != LABEL_STANDARD_INPUT && color_hl != NULL)
+  {
+    str(color_hl);
+    str(color_st);
+  }
+
   if (!partname.empty())
   {
-    if (pathname == NULL)
-      chr(' ');
     chr('{');
     str(partname);
-    chr('}');
+    str("} ");
   }
+
   str(color_off);
+
   str(" matches");
   nl();
+
   mode_ |= BINARY;
 }
 
@@ -677,7 +759,7 @@ void Output::format(const char *format, size_t matches)
 }
 
 // output formatted match with options --format, --format-open, --format-close
-void Output::format(const char *format, const char *& pathname, const std::string& partname, size_t matches, size_t *matching, reflex::AbstractMatcher *matcher, bool body, bool next)
+void Output::format(const char *format, const char *pathname, const std::string& partname, size_t matches, size_t *matching, reflex::AbstractMatcher *matcher, bool& heading, bool body, bool next)
 {
   if (!body)
     lineno_ = 0;
@@ -721,7 +803,7 @@ void Output::format(const char *format, const char *& pathname, const std::strin
         {
           if (flag_with_filename)
           {
-            if (pathname != NULL)
+            if (heading)
             {
               if (arg != NULL)
                 str(arg, s - arg - 1);
@@ -739,11 +821,11 @@ void Output::format(const char *format, const char *& pathname, const std::strin
         break;
 
       case 'F':
-        if (flag_with_filename && (pathname != NULL || !partname.empty()))
+        if (flag_with_filename && (heading || !partname.empty()))
         {
           if (arg != NULL)
             str(arg, s - arg - 1);
-          if (pathname != NULL)
+          if (heading)
             str(pathname);
           if (!partname.empty())
           {
@@ -761,7 +843,7 @@ void Output::format(const char *format, const char *& pathname, const std::strin
         break;
 
       case 'f':
-        if (pathname != NULL)
+        if (heading)
         {
           str(pathname);
           if (!partname.empty())
@@ -774,7 +856,7 @@ void Output::format(const char *format, const char *& pathname, const std::strin
         break;
 
       case 'a':
-        if (pathname != NULL)
+        if (heading)
         {
           const char *basename = strrchr(pathname, PATHSEPCHR);
           if (basename == NULL)
@@ -785,7 +867,7 @@ void Output::format(const char *format, const char *& pathname, const std::strin
         break;
 
       case 'p':
-        if (pathname != NULL)
+        if (heading)
         {
           const char *basename = strrchr(pathname, PATHSEPCHR);
           if (basename != NULL)
@@ -798,7 +880,7 @@ void Output::format(const char *format, const char *& pathname, const std::strin
         break;
 
       case 'H':
-        if (flag_with_filename && (pathname != NULL || !partname.empty()))
+        if (flag_with_filename && (heading || !partname.empty()))
         {
           if (arg != NULL)
             str(arg, s - arg - 1);
@@ -812,7 +894,7 @@ void Output::format(const char *format, const char *& pathname, const std::strin
             name.push_back('}');
             quote(name.c_str(), name.size());
           }
-          else if (pathname != NULL)
+          else if (heading)
           {
             quote(pathname, strlen(pathname));
           }
@@ -827,14 +909,14 @@ void Output::format(const char *format, const char *& pathname, const std::strin
         if (!partname.empty())
         {
           std::string name;
-          if (pathname != NULL)
+          if (heading)
             name = pathname;
           name.push_back('{');
           name.append(partname);
           name.push_back('}');
           quote(name.c_str(), name.size());
         }
-        else if (pathname != NULL)
+        else if (heading)
         {
           quote(pathname, strlen(pathname));
         }
@@ -1239,11 +1321,11 @@ void Output::format(const char *format, const char *& pathname, const std::strin
 
   // only output the pathname once
   if (flag_heading)
-    pathname = NULL;
+    heading = false;
 }
 
 // output formatted match with options -v --format
-void Output::format_invert(const char *format, const char *& pathname, const std::string& partname, size_t matches, size_t lineno, size_t offset, const char *ptr, size_t size, bool next)
+void Output::format_invert(const char *format, const char *pathname, const std::string& partname, size_t matches, size_t lineno, size_t offset, const char *ptr, size_t size, bool& heading, bool next)
 {
   size_t len = 0;
   const char *sep = NULL;
@@ -1279,7 +1361,7 @@ void Output::format_invert(const char *format, const char *& pathname, const std
         {
           if (flag_with_filename)
           {
-            if (pathname != NULL)
+            if (heading)
             {
               if (arg != NULL)
                 str(arg, s - arg - 1);
@@ -1297,11 +1379,11 @@ void Output::format_invert(const char *format, const char *& pathname, const std
         break;
 
       case 'F':
-        if (flag_with_filename && (pathname != NULL || !partname.empty()))
+        if (flag_with_filename && (heading || !partname.empty()))
         {
           if (arg != NULL)
             str(arg, s - arg - 1);
-          if (pathname != NULL)
+          if (heading)
             str(pathname);
           if (!partname.empty())
           {
@@ -1319,7 +1401,7 @@ void Output::format_invert(const char *format, const char *& pathname, const std
         break;
 
       case 'f':
-        if (pathname != NULL)
+        if (heading)
         {
           str(pathname);
           if (!partname.empty())
@@ -1332,7 +1414,7 @@ void Output::format_invert(const char *format, const char *& pathname, const std
         break;
 
       case 'a':
-        if (pathname != NULL)
+        if (heading)
         {
           const char *basename = strrchr(pathname, PATHSEPCHR);
           if (basename == NULL)
@@ -1343,7 +1425,7 @@ void Output::format_invert(const char *format, const char *& pathname, const std
         break;
 
       case 'p':
-        if (pathname != NULL)
+        if (heading)
         {
           const char *basename = strrchr(pathname, PATHSEPCHR);
           if (basename != NULL)
@@ -1356,21 +1438,21 @@ void Output::format_invert(const char *format, const char *& pathname, const std
         break;
 
       case 'H':
-        if (flag_with_filename && (pathname != NULL || !partname.empty()))
+        if (flag_with_filename && (heading || !partname.empty()))
         {
           if (arg != NULL)
             str(arg, s - arg - 1);
           if (!partname.empty())
           {
             std::string name;
-            if (pathname != NULL)
+            if (heading)
               name = pathname;
             name.push_back('{');
             name.append(partname);
             name.push_back('}');
             quote(name.c_str(), name.size());
           }
-          else if (pathname != NULL)
+          else if (heading)
           {
             quote(pathname, strlen(pathname));
           }
@@ -1385,14 +1467,14 @@ void Output::format_invert(const char *format, const char *& pathname, const std
         if (!partname.empty())
         {
           std::string name;
-          if (pathname != NULL)
+          if (heading)
             name = pathname;
           name.push_back('{');
           name.append(partname);
           name.push_back('}');
           quote(name.c_str(), name.size());
         }
-        else if (pathname != NULL)
+        else if (heading)
         {
           quote(pathname, strlen(pathname));
         }
@@ -1614,7 +1696,7 @@ void Output::format_invert(const char *format, const char *& pathname, const std
 
   // only output the pathname once
   if (flag_heading)
-    pathname = NULL;
+    heading = false;
 }
 
 // output a quoted string with escapes for \ and "
